@@ -14,7 +14,7 @@
 class QLabel;
 class QPainter;
 class KToggleAction;
-class QPrinter;
+class KPrinter;
 
 
 #include <kmultipage.h>
@@ -33,7 +33,7 @@ public:
   void scale(int w, int h, bool anti);
   QPixmap scaledPixmap();
 
-  void print(QPrinter *p);
+  void print(KPrinter *p);
 
 
 private:
