@@ -1,8 +1,11 @@
+#include <qfileinfo.h>
 #include <qframe.h>
 #include <qlayout.h>
 
 #include <kaction.h>
+#include <kapp.h>
 #include <kconfig.h>
+#include <kdebug.h>
 #include <kdirwatch.h>
 #include <kglobalsettings.h>
 #include <kiconloader.h>
@@ -12,6 +15,9 @@
 
 #include "kgv_view.h"
 #include "kgv_miniwidget.h"
+#include "kgvpagedecorator.h"
+#include "kgvpageview.h"
+#include "kgvmainwidget.h"
 #include "kpswidget.h"
 #include "marklist.h"
 #include "scrollbox.h"
@@ -59,11 +65,21 @@ KGVPart::KGVPart( bool bBrowserView, QWidget *parentWidget,
  :  KParts::ReadOnlyPart( parent, name )
 {
     setInstance( KGVFactory::instance() );
-   
+
+    // Don't show the progress info dialog if we're embedded in Konqueror.
+    showProgressInfo( !bBrowserView );
+
+    // setenv( "GS_LIB", "/usr/local/kde/share/apps/kghostview", true );
+
+    _watch = new KDirWatch;
+    connect( _watch, SIGNAL( fileDirty( const QString& ) ),
+	     this, SLOT( slotFileDirty( const QString& ) ) );
     
     // Setup main widget
-    _mainWidget = new QWidget( parentWidget );
+    _mainWidget = new KGVMainWidget( parentWidget );
     _mainWidget->setFocusPolicy( QWidget::StrongFocus );
+    connect( _mainWidget, SIGNAL( spacePressed() ),
+             this, SLOT( slotReadDown() ) );
 
     QHBoxLayout* hlay = new QHBoxLayout( _mainWidget, 0, 0 );
     QVBoxLayout* vlay = new QVBoxLayout( hlay );
@@ -75,8 +91,6 @@ KGVPart::KGVPart( bool bBrowserView, QWidget *parentWidget,
 
     _markList = new MarkList( _mainWidget, "marklist" );
     _markList->setMinimumWidth( PAGELIST_WIDTH );
-    _markList->setSelectColors( _mainWidget->colorGroup().highlight(), 
-				_mainWidget->colorGroup().highlightedText());
     vlay->addWidget( _markList, 1 );
 
     _divider = new QFrame( _mainWidget, "divider" );
@@ -85,16 +99,19 @@ KGVPart::KGVPart( bool bBrowserView, QWidget *parentWidget,
     _divider->setMinimumWidth( 3 );
     hlay->addWidget( _divider );
  
-    _psView = new KPSView( _mainWidget, "psview" );
-    hlay->addWidget( _psView, 1);
-    _mainWidget->setFocusProxy( _psView );
+    _pageView = new KGVPageView( _mainWidget, "pageview" );
+    _pageView->viewport()->setBackgroundMode( QWidget::PaletteMid );
+    hlay->addWidget( _pageView, 1 );
+    _mainWidget->setFocusProxy( _pageView );
 
     setWidget( _mainWidget );
 
-    
-    // Setup document manager
-    _docManager = new KGVMiniWidget( _psView->psWidget(), this );
-
+    KGVPageDecorator* pageDeco = new KGVPageDecorator( _pageView->viewport() );
+    pageDeco->hide();
+    KPSWidget* psWidget = new KPSWidget( pageDeco );
+    _pageView->setPage( pageDeco );
+    _docManager = new KGVMiniWidget( this );
+    _docManager->setPSWidget( psWidget );
     
     // Connect up the necessary signals and slots.
     connect( _markList, SIGNAL( selected( int ) ), 
@@ -103,12 +120,12 @@ KGVPart::KGVPart( bool bBrowserView, QWidget *parentWidget,
 	     _markList, SLOT( select( int ) ) );
 
     connect( _scrollBox, SIGNAL( valueChangedRelative( int, int ) ),
-	     _psView, SLOT( scrollBy( int, int ) ) );
-    connect( _psView->psWidget(), SIGNAL( pageSizeChanged( QSize ) ),
+	     _pageView, SLOT( scrollBy( int, int ) ) );
+    connect( _pageView, SIGNAL( pageSizeChanged( QSize ) ),
 	     _scrollBox, SLOT( setPageSize( QSize ) ) );
-    connect( _psView, SIGNAL( viewSizeChanged( QSize ) ),
+    connect( _pageView, SIGNAL( viewSizeChanged( QSize ) ),
 	     _scrollBox, SLOT( setViewSize( QSize ) ) );
-    connect( _psView, SIGNAL( contentsMoving( int, int ) ),
+    connect( _pageView, SIGNAL( contentsMoving( int, int ) ),
 	     _scrollBox, SLOT( setViewPos( int, int ) ) );
 
     connect( _docManager, SIGNAL( fileChangeFailed() ),
@@ -121,10 +138,8 @@ KGVPart::KGVPart( bool bBrowserView, QWidget *parentWidget,
   
   
     // File Menu
-    if ( !bBrowserView )
-	statedep->insert( KStdAction::saveAs( miniWidget(), SLOT( saveAs() ), 
+    statedep->insert( KStdAction::saveAs( miniWidget(), SLOT( saveAs() ), 
 			    actionCollection() ) );
-  
     statedep->insert( new KAction( i18n( "Document &Info..." ), 0, 
 			    miniWidget(), SLOT( info() ),
 			    actionCollection(), "info") );
@@ -144,6 +159,7 @@ KGVPart::KGVPart( bool bBrowserView, QWidget *parentWidget,
     connect( orientation, SIGNAL( activated( int ) ),
 	     this, SLOT( slotOrientation( int ) ) );
   
+    //TODO -- zoom (maybe remove media from toolbar)
     media = new KSelectAction( i18n( "Paper &Size" ), 0, 0, 0,
 		       actionCollection(), "media_menu" );
     connect( media, SIGNAL( activated( int ) ),
@@ -169,8 +185,13 @@ KGVPart::KGVPart( bool bBrowserView, QWidget *parentWidget,
 			CTRL+Key_End, this, SLOT( slotGotoEnd() ), 
 			actionCollection(), "goToEnd" );
 		    
-    endDocAct	= new KAction( i18n( "Read down document" ), "next",
-			Key_Space, this, SLOT( slotReadDown() ),
+    readUpAct	= new KAction( i18n( "Read up document" ), "previous",
+			SHIFT+Key_Space, this, SLOT( slotReadUp() ),
+			actionCollection(), "readUp" );
+
+    // don't specify Key_Space as Accel, it breaks the lineedit in konq! (Simon)
+    readDownAct	= new KAction( i18n( "Read down document" ), "next",
+			/*Key_Space*/0, this, SLOT( slotReadDown() ),
 			actionCollection(), "readDown" );
 
     gotoAct	= KStdAction::gotoPage( _docManager, SLOT( goToPage() ),
@@ -184,16 +205,16 @@ KGVPart::KGVPart( bool bBrowserView, QWidget *parentWidget,
     statedep->insert( forwardAct );
     statedep->insert( startAct );
     statedep->insert( endAct );
-    statedep->insert( endDocAct );
+    statedep->insert( readUpAct );
+    statedep->insert( readDownAct );
     statedep->insert( gotoAct);
 
 
-    // Pagemarks Menu
-    markAct = new KAction(i18n( "Mark Current Page" ), "flag",
-		    Key_Enter, _docManager, SLOT( markPage() ), 
-		    actionCollection(), "mark_current");
-    statedep->insert( markAct );
-    
+    // Page Marks Menu
+    statedep->insert(
+	    new KAction( i18n( "Mark Current Page" ), "flag", CTRL+Key_M, 
+		    _markList, SLOT( markCurrent() ), 
+		    actionCollection(), "mark_current" ) );
     statedep->insert(  
 	    new KAction( i18n( "Mark &All Pages" ), 0, 
 		    _markList, SLOT( markAll() ),
@@ -237,21 +258,21 @@ KGVPart::KGVPart( bool bBrowserView, QWidget *parentWidget,
 			actionCollection(), "fancy_page_labels" );
     fancyAct->setChecked( _docManager->areFancyPageLabelsEnabled() );
     
-    new KAction( i18n( "Configure &Interpreter..." ), 0, 
+    new KAction( i18n( "&Configure KGhostView..." ), "configure", 0, 
 		miniWidget(), SLOT( configureGhostscript() ),
-		actionCollection(), "config_interpreter" );
+		actionCollection(), "configure" );
 
     //TODO
     // progress bars for conversions (see loadingProgress())
   
-    _watch = 0;
-  
     _extension = new KGVBrowserExtension( this );
   
     setXMLFile( "kgv_part.rc" );
   
     connect( miniWidget(), SIGNAL( newPageShown( int ) ),
 	     this, SLOT( slotNewPage( int ) ) );
+    connect( _pageView, SIGNAL( contentsMoving( int, int ) ),
+	     this, SLOT( slotPageMoved( int, int ) ) );
 
     readSettings();
 
@@ -260,20 +281,27 @@ KGVPart::KGVPart( bool bBrowserView, QWidget *parentWidget,
 
 KGVPart::~KGVPart()
 {
-    stopWatching();
+    delete _watch;
     writeSettings();
 }
 
+bool KGVPart::closeURL()
+{
+    _watch->removeDir( QFileInfo( m_file ).dirPath() );
+    return KParts::ReadOnlyPart::closeURL();
+}
+
 void
 KGVPart::writeSettings()
 {
   KConfig *config = KGVFactory::instance()->config();
-
+  QString oldGroup = config->group();
   config->setGroup( "General" );
   config->writeEntry ("ShowScrollBars", showscrollbars->isChecked());
   config->writeEntry ("WatchFile", watchaction->isChecked());
   config->writeEntry ("ShowPageList", showmarklist->isChecked());
   config->writeEntry ("ShowPageNames", fancyAct->isChecked());
+  config->setGroup( oldGroup );
 
   miniWidget()->writeSettings();
 
@@ -284,7 +312,7 @@ void
 KGVPart::readSettings()
 {
     KConfig *config = KGVFactory::instance()->config();
-
+    QString oldGroup = config->group();
     config->setGroup( "General" );
 
     showscrollbars->setChecked( 
@@ -299,97 +327,87 @@ KGVPart::readSettings()
   
     fancyAct->setChecked( config->readBoolEntry( "ShowPageNames", false ) );
     slotFancy();
+
+    config->setGroup( oldGroup );
 }
 
 void KGVPart::slotScrollLeft()
 {
-    _psView->scrollLeft();
+    _pageView->scrollLeft();
 }
 
 void KGVPart::slotScrollRight()
 {
-    _psView->scrollRight();
+    _pageView->scrollRight();
 }
 
 void KGVPart::slotScrollUp()
 {
-    _psView->scrollUp();
+    _pageView->scrollUp();
 }
 
 void KGVPart::slotScrollDown()
 {
-    _psView->scrollDown();
+    _pageView->scrollDown();
+}
+
+void KGVPart::slotReadUp()
+{
+    if( !_pageView->readUp() ) {
+	_docManager->prevPage();
+	_pageView->scrollBottom();
+    }	
 }
 
 void KGVPart::slotReadDown()
 {
-    if( !_psView->readDown() )
+    if( !_pageView->readDown() )
 	slotNextPage();
 }
 
 void KGVPart::slotPrevPage()
 {
     _docManager->prevPage();
-    _psView->scrollTop();
+    _pageView->scrollTop();
 }
 
 void KGVPart::slotNextPage()
 {
-    _docManager->nextPage();
-    _psView->scrollTop();
+    if( _docManager->nextPage() ) {
+	_pageView->scrollTop();
+    }
+    else
+	_pageView->scrollBottom();
 }
 
 void KGVPart::slotGotoStart()
 {
     _docManager->firstPage();
-    _psView->scrollTop();
+    _pageView->scrollTop();
 }
 
 void
 KGVPart::slotGotoEnd()
 {
     _docManager->lastPage();
-    _psView->scrollTop();
-}
-
-void KGVPart::startWatching()
-{
-    //NOTE: isFileOpen() will (appropriately) return false if the viewer is
-     // viewing data from stdin
-    if( miniWidget()->isFileOpen() ) {
-	// Setup the dirwatcher to watch this file.
-	_watch = new KDirWatch();
-	_watch->addDir( m_file );
-	connect( _watch, SIGNAL( dirty( const QString& ) ),
-		 _docManager, SLOT( fileChanged( const QString& ) ) );
-	_watch->startScan();
-    }
-}
-
-void KGVPart::stopWatching()
-{
-    if( _watch != 0 ) {
-	delete _watch;
-	_watch = 0;
-    }
+    _pageView->scrollTop();
 }
 
 void KGVPart::slotWatchFile()
 {
     if( watchaction->isChecked() )
-	startWatching();
+	_watch->startScan();
     else
-	stopWatching();
+	_watch->stopScan();
 }
 
 void KGVPart::slotCancelWatch()
 {
-    stopWatching();
+    _watch->stopScan();
     watchaction->setChecked( false );
 }
 
-void
-KGVPart::enableStateDepActions( bool enable )
+void KGVPart::enableStateDepActions( bool enable )
 {
     int count = statedep->count();
     for ( int i = 0; i < count; i++ )
@@ -402,63 +420,122 @@ KGVPart::enableStateDepActions( bool enable )
     }
 }
 
-bool
-KGVPart::openStdin()
-{
-  QString s ("-");
-  bool b = _docManager->openFile(s);
-  if (b)
-    enableStateDepActions( true );  //do we want eAA (b) ?
-  return b;
-}
-
-bool
-KGVPart::openFile()
-{
-    bool success = _docManager->openFile( m_file );
-    
-    if( success ) {
-	enableStateDepActions( true );
-	slotWatchFile();
+void KGVPart::updatePageDepActions()
+{    
+    if( _docManager->atFirstPage() ) {
+	backAct->setEnabled( false );
+	startAct->setEnabled( false );
+    }
+    else {
+	backAct->setEnabled( true );
+	startAct->setEnabled( true );
     }
 
+    if( _docManager->atLastPage() ) {
+	forwardAct->setEnabled( false );
+	endAct->setEnabled( false );
+    }
+    else {
+	forwardAct->setEnabled( true );
+	endAct->setEnabled( true );
+    }
+
+    updateReadUpDownActions();
+}
+
+void KGVPart::updateReadUpDownActions()
+{
+    if( _docManager->atFirstPage() && _pageView->atTop() ) 
+	readUpAct->setEnabled( false );
+    else
+	readUpAct->setEnabled( true );
+    
+    if( _docManager->atLastPage() && _pageView->atBottom() ) 
+	readDownAct->setEnabled( false );
+    else
+	readDownAct->setEnabled( true );
+}
+
+bool KGVPart::openStdin()
+{
+    bool success = _docManager->openFile( "-" );
+    if( success ) {
+	_docManager->firstPage();	
+	enableStateDepActions( true );  //do we want eAA (b) ?
+    }
     return success;
 }
 
-void
-KGVPart::slotFancy()
+bool KGVPart::openFile()
+{
+    bool success = _docManager->openFile( m_file );
+    if( success ) {
+	_docManager->firstPage();	
+	enableStateDepActions( true );
+	_watch->addDir( QFileInfo( m_file ).dirPath() );
+	slotWatchFile();
+    }
+    return success;
+}
+
+void KGVPart::slotFancy()
 {
     _docManager->enableFancyPageLabels( fancyAct->isChecked() );
 }
 
-void
-KGVPart::slotNewPage( int )
+void KGVPart::slotFileDirty( const QString& fileName )
 {
+    if( fileName == m_file )
+	_docManager->fileChanged( fileName );
+}
+
+void KGVPart::slotNewPage( int )
+{
+    updatePageDepActions();
   ;//media->setCurrentItem (miniWidget()->getSize()-1);
   ;//orientation->setCurrentItem (miniWidget()->getOrientation()-1);
   //TODO -- zoom
 }
 
+void KGVPart::slotPageMoved( int, int )
+{
+    updateReadUpDownActions();
+}
+
 void KGVPart::slotOrientation( int id )
 {
-    if( id == 0 )
-	miniWidget()->setAutoOrientation();
-    else
-	miniWidget()->setManualOrientation( id );
+    switch( id ) {
+    case 0: 
+	miniWidget()->restoreOverrideOrientation();
+	break;
+    case 1: 
+	miniWidget()->setOverrideOrientation( KGV::PORTRAIT );
+	break;
+    case 2: 
+	miniWidget()->setOverrideOrientation( KGV::LANDSCAPE );
+	break;
+    case 3: 
+	miniWidget()->setOverrideOrientation( KGV::UPSIDEDOWN );
+	break;
+    case 4: 
+	miniWidget()->setOverrideOrientation( KGV::SEASCAPE );
+	break;
+    default: ;
+    }
 }
 
 void
-KGVPart::slotMedia(int id)
+KGVPart::slotMedia( int id )
 {
     if( id == 0 )
-	miniWidget()->setAutoPageMedia();
+	miniWidget()->restoreOverridePageMedia();
     else
-	miniWidget()->setManualPageMedia( id );
+	miniWidget()->setOverridePageMedia( id - 1 );
 }
 
 void KGVPart::slotShowScrollBars()
 {
-    _psView->enableScrollBars( showscrollbars->isChecked() );
+    _pageView->enableScrollBars( showscrollbars->isChecked() );
 }
 
 void
@@ -480,6 +557,7 @@ KGVPart::slotShowMarkList()
 KGVBrowserExtension::KGVBrowserExtension( KGVPart *parent ) :
   KParts::BrowserExtension( parent, "KGVBrowserExtension" )
 {
+    emit enableAction( "print", true );
 }
 
 void
