@@ -25,16 +25,13 @@
 #include <qpixmap.h>
 #include <qimage.h>
 #include <qbitmap.h>
-#include <qfiledlg.h> 
-#include <qbttngrp.h>
+#include <qfiledialog.h> 
+#include <qbuttongroup.h>
 #include <qbutton.h>
-#include <qradiobt.h>
-#include <qlayout.h>
+#include <qradiobutton.h>
 #include <qlistbox.h>
-#include <kiconloader.h>
-#include <kapp.h>
-#include <kintegerline.h>
-#include <kwizard.h>
+#include <qwizard.h>
+
 #include "kresize.h"
 
 class KIconListBox;
@@ -107,10 +104,9 @@ protected slots:
   void buttonClicked(int);
 
 protected:
-  KWizard *wiz;
+  QWizard *wiz;
   QButtonGroup *grp;
   QRadioButton *rbscratch, *rbtempl;
-  QBoxLayout *ml, *l;
 };
 
 class NewFromTemplate : public QWidget
@@ -127,12 +123,11 @@ public slots:
  
 protected:
   KIconListBox *templates;
-  KWizard *wiz;
-  QBoxLayout *ml, *l;
+  QWizard *wiz;
   QGroupBox *grp;
 };
 
-class KNewIcon : public KWizard
+class KNewIcon : public QWizard
 {
   Q_OBJECT
 public:
@@ -156,14 +151,11 @@ signals:
   void newicon(const QString);
 
 protected:
-  //static QList<KIconTemplate> templatelist;
-  //QButtonGroup *bgrptmpl, *bgrpstl;
   NewSelect       *select;
-  //NewFromScratch  *scratch;
-  KResize  *scratch;
+  KResizeWidget   *scratch;
   NewFromTemplate *templ;
-  KWizardPage *p1, *p2;
-  int openstyle;
+  QWidget         *act;
+  int              openstyle;
 };
 
 #endif // __KNEWICON_H__
