@@ -1,5 +1,5 @@
 // -*- c++ -*-
-// $Id: ellipse.h,v 1.6 1999/06/19 23:17:04 kulow Exp $
+// $Id: ellipse.h,v 1.7 2000/08/06 10:25:10 bieker Exp $
 
 #ifndef ELLIPSE_H
 #define ELLIPSE_H
@@ -13,7 +13,7 @@ class Ellipse : public Tool
   Q_OBJECT
 public:
   Ellipse();
-  Ellipse(const char *toolname);
+  Ellipse(const QString & toolname);
   void mousePressEvent(QMouseEvent *e);
   void mouseMoveEvent(QMouseEvent *e);
   void mouseReleaseEvent(QMouseEvent *e);
