@@ -25,9 +25,6 @@ static KCmdLineOptions options[] =
    { 0, 0, 0 }
 };
 
-int openwins= 0;
-
-
 int main( int argc, char **argv )
 {
     KAboutData aboutData(APPNAME, I18N_NOOP("KPaint"), 
@@ -56,18 +53,18 @@ int main( int argc, char **argv )
         KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
         for(int i = 0; i < args->count(); i++)
         {
-            kp = new KPaint(QFile::decodeName(args->arg(i)));
+            kp = new KPaint(args->url(i));
+            kp->show();
         }
 
         if (args->count() == 0)
         {
             kp = new KPaint();
+            kp->show();
         }
         
         args->clear();
 
-        if(kp != NULL)
-            kp->show();
     }
 
     QObject::connect(&kpaintApp, SIGNAL(lastWindowClosed()),
