@@ -1,7 +1,7 @@
 /*
 * canvas.cpp -- Implementation of class KImageCanvas.
 * Author:       Sirtaj Singh Kang
-* Version:      $Id: canvas.cpp,v 1.27 2001/02/18 13:02:58 mkretz Exp $
+* Version:      $Id: canvas.cpp,v 1.34 2001/05/30 19:15:24 mkretz Exp $
 * Generated:    Thu Oct  9 09:03:54 EST 1997
 */
 
@@ -25,6 +25,7 @@
 #include <dcopclient.h>
 #include <kapp.h>
 #include <kstddirs.h>
+#include <ktempfile.h>
 
 #include <X11/Xlib.h>
 
@@ -38,23 +39,60 @@ KImageCanvas::KImageCanvas( QWidget *parent )
   _status( OK ),
   _file( "" ),
   _client( 0 ),
-  _orig( 0 )
+  _orig( 0 ),
+  m_centered( false ),
+  m_depth( 0 )
 {
-  _client = new KVImageHolder( this );
-  
-  connect(_client, SIGNAL( contextPress( const QPoint& ) ), 
-    SIGNAL ( contextPress( const QPoint& ) ) );
-        connect(_client, SIGNAL( selected( bool ) ),
-                SIGNAL( selected( bool ) ) );
-  addChild( _client );
+  _client = new KVImageHolder( viewport() );
+  addChild( _client, 0, 0 );
+
+  connect(_client, SIGNAL( contextPress( const QPoint& ) ), SIGNAL( contextPress( const QPoint& ) ) );
+  connect(_client, SIGNAL( selected( bool ) ), SIGNAL( selected( bool ) ) );
 
-  _client->setAutoResize( TRUE );
   _client->hide();
 }
 
 KImageCanvas::~KImageCanvas()
 {
-  delete _orig;
+  if( _orig )
+    delete _orig;
+  _orig = 0;
+}
+
+void KImageCanvas::setBgColor( const QColor & color )
+{
+  viewport()->setBackgroundColor( color );
+}
+
+const QColor & KImageCanvas::bgColor() const
+{
+  return viewport()->backgroundColor();
+}
+
+int KImageCanvas::pixmapDepth() const
+{
+  return m_depth;
+}
+
+QSize KImageCanvas::originalSize() const
+{
+  if( isEmpty() )
+    return QSize( 0, 0 );
+
+  return m_originalsize;
+  /*if( _orig )
+    return _orig->size();
+  else
+    return _client->imagePix()->size();*/
+}
+
+void KImageCanvas::slot_setCentered( bool centered )
+{
+  m_centered = centered;
+  if( m_centered )
+    moveChild( _client, ( viewport()->width() - _client->width() ) / 2, ( viewport()->height() - _client->height() ) / 2 );
+  else
+    moveChild( _client, 0, 0 );
 }
 
 KImageCanvas::Error KImageCanvas::status() const
@@ -92,12 +130,15 @@ int KImageCanvas::load( const QString & file, const QString & URL, bool max )
 
   QApplication::setOverrideCursor( WaitCursor );
 
-  QPixmap newImage;
+  QImage newImage;
   bool loadOK = newImage.load( realfile );
 
   if( !loadOK && realfile.contains( ".tga", false ) ) {
     loadOK = newImage.load( realfile, "TGA" );
   }
+  m_depth = newImage.depth();
+  QPixmap newPixmap;
+  newPixmap.convertFromImage( newImage, ColorOnly );
 
   QColor::leaveAllocContext();
 
@@ -106,23 +147,39 @@ int KImageCanvas::load( const QString & file, const QString & URL, bool max )
   }
 
   if( loadOK ) {
-    delete _orig; // delete old buffered image
-    _orig = 0;
+    // delete old buffered image
+    forgetOriginal();
 
     //remove the contents
     removeChild( _client );
 
     //put the new image into the _client
-    _client->move( 0, 0 );
-    _originalSize = size();
+    m_originalsize = newPixmap.size();
     if ( max ) {
-      maxpect( this, &newImage );
+      _orig = new QPixmap( newPixmap );
+      maxpect( this, &newPixmap );
     }
-    _client->setImagePix( newImage );
-    _client->show();
+
+    KVImageHolder *newclient = new KVImageHolder();
+    connect(newclient, SIGNAL( contextPress( const QPoint& ) ), SIGNAL( contextPress( const QPoint& ) ) );
+    connect(newclient, SIGNAL( selected( bool ) ), SIGNAL( selected( bool ) ) );
+
+    newclient->setImagePix( newPixmap );
 
     //and readd the contents again
-    addChild( _client );
+    if( m_centered )
+    {
+      int x = QMAX( 0, ( viewport()->width() - newclient->width() ) / 2 );
+      int y = QMAX( 0, ( viewport()->height() - newclient->height() ) / 2 );
+      addChild( newclient, x, y );
+    }
+    else
+      addChild( newclient, 0, 0 );
+
+    newclient->show();
+    _client->hide();
+    delete _client;
+    _client = newclient;
     
     updateScrollBars();
 
@@ -172,8 +229,12 @@ void KImageCanvas::forgetOriginal()
 
 void KImageCanvas::clear()
 {
-  _client->setBackgroundColor( QColor(0,0,0) );
-  _client->resize( 0, 0 );
+  _client->clear();
+}
+
+bool KImageCanvas::isEmpty() const
+{
+  return ! _client->imagePix();
 }
 
 void KImageCanvas::transformImage( const QWMatrix& mat )
@@ -193,23 +254,25 @@ void KImageCanvas::transformImage( const QWMatrix& mat )
   QApplication::restoreOverrideCursor();
 }
 
-void KImageCanvas::tileToDesktop() const
+void KImageCanvas::slot_tileToDesktop() const
 {
   setDesktopBackground( Tiled );
 }
 
-void KImageCanvas::maxToDesktop() const
+void KImageCanvas::slot_maxToDesktop() const
 {
   setDesktopBackground( Scaled );
 }
 
-void KImageCanvas::maxpectToDesktop() const
+void KImageCanvas::slot_maxpectToDesktop() const
 {
   setDesktopBackground( CentredMaxpect );
 }
 
 void KImageCanvas::setDesktopBackground( WallpaperMode mode ) const
 {
+  QApplication::setOverrideCursor( waitCursor );
+
   if( _client->imagePix() == 0 )
     return;
   
@@ -237,16 +300,20 @@ void KImageCanvas::setDesktopBackground( WallpaperMode mode ) const
   arg << (int)mode;
 
   client->send( appname, "KBackgroundIface", "setWallpaper(QString,int)", data );
+
+  QApplication::restoreOverrideCursor();
 }
 
 static void maxpect( QWidget *dest, QPixmap *image )
 {
+  QApplication::setOverrideCursor( WaitCursor );
   double dh = double( dest->height() ) / double( image->height() );
   double dw = double( dest->width() ) / double( image->width() );
   double d = ( dh < dw ? dh : dw );
   
   image->convertFromImage(image->convertToImage().smoothScale(
       int( d*image->width() ), int( d*image->height() ) ) );
+  QApplication::restoreOverrideCursor();
 }
 
 void KImageCanvas::resizeEvent( QResizeEvent *ev )
@@ -261,7 +328,7 @@ QImage KImageCanvas::getImage() const
   return _client->imagePix()->convertToImage();
 }
 
-void KImageCanvas::setImage( const QImage& image )
+void KImageCanvas::slot_setImage( const QImage& image )
 {
   transPixmap();
 
@@ -288,7 +355,7 @@ QPixmap *KImageCanvas::transPixmap()
   return client;
 }
 
-void KImageCanvas::cropImage()
+void KImageCanvas::slot_cropImage()
 {
   QRect select = _client->selected();
 
@@ -318,51 +385,48 @@ void KImageCanvas::cropImage()
 static QString loadStdin()
 {
   if( feof( stdin ) ) {
-    return "";
+    return QString::null;
   }
 
-  char name[100];
-  if( tmpnam( name ) == 0  ) {
-    return "";
-  }
+  KTempFile tempfile;
 
   char buffer[ BUF_SIZE ];
-  FILE *o;
+  FILE *o = tempfile.fstream();
 
-  o = fopen( name, "w" );
   if( o == 0 ) {
-    return "";
+    return QString::null;
   }
 
   while( !feof( stdin ) ) {
-    size_t bytes = fread( buffer, sizeof(char), BUF_SIZE,
-      stdin );
+    size_t bytes = fread( buffer, sizeof(char), BUF_SIZE, stdin );
 
     fwrite( buffer, sizeof( char ), bytes, o );
   }
-  fclose( o );
+  tempfile.close();
 
-  return name;
+  return tempfile.name();
 }
 
 
-void KImageCanvas::maxToWin()
+void KImageCanvas::slot_maxToWin()
 {
   transPixmap();
 
   if( _client->imagePix() == 0 )
     return;
   
-  QPixmap image = *(_client->imagePix());
+  QPixmap image = *_orig;
 
+  QApplication::setOverrideCursor( WaitCursor );
   image.convertFromImage(image.convertToImage().smoothScale(
       width(),height()));
   _client->setImagePix( image );
+  QApplication::restoreOverrideCursor();
   
   emit imageSizeChanged();
 }
 
-void KImageCanvas::maxpectToWin()
+void KImageCanvas::slot_maxpectToWin()
 {
   transPixmap();
 
@@ -377,45 +441,30 @@ void KImageCanvas::maxpectToWin()
   emit imageSizeChanged();
 }
 
-void KImageCanvas::mousePressEvent( QMouseEvent *ev )
+void KImageCanvas::viewportMousePressEvent( QMouseEvent *ev )
 {
   if ( ev->button() == RightButton )
     emit contextPress( mapToGlobal( ev->pos() ) );
-
-  QScrollView::mousePressEvent( ev );
 }
 
 void KImageCanvas::copyImage( QPaintDevice *dest ) const
 {
+  // only print if there is an image
+  if( isEmpty() )
+    return;
   QPainter painter( dest );
   painter.drawPixmap( 0, 0, *(_client->imagePix()) );
 }
 
-void KImageCanvas::mouseReleaseEvent( QMouseEvent *ev )
-{
-  // wheel support
-  // maybe one day this will work...
-
-#if 0
-  switch ( ev->button() ) {
-    case 4: lineUp();  break;
-    case 5: lineDown();  break;
-    default:  
-      QScrollView::mouseReleaseEvent( ev );
-  }
-#else
-  QScrollView::mouseReleaseEvent( ev );
-#endif
-
-}
-
 /*
 ******************************
 * KVImageHolder implementation
 */
 
 KVImageHolder::KVImageHolder( QWidget *parent )
-    : QLabel( parent ), _selected( false ),
+    : QLabel( parent ),
+    _selected( false ),
+    m_empty( true ),
     _painter( new QPainter ),
     _pen( new QPen( QColor( 255, 255, 255 ), 0, DashLine ) )
 {
@@ -514,9 +563,12 @@ QRect KVImageHolder::selected() const
 void KVImageHolder::setImagePix( QPixmap pix )
 {
   clearSelection();
-        const QBitmap nullBitmap;
-        pix.setMask( nullBitmap ); //don't show transparency
+
+  const QBitmap nullBitmap;
+  pix.setMask( nullBitmap ); //don't show transparency
+
   setPixmap( pix );
+  m_empty = false;
   resize( pix.size() );
 }
 
@@ -524,15 +576,26 @@ void KVImageHolder::setImageMovie( const QMovie &movie )
 {
   clearSelection();
   setMovie( movie );
+  m_empty = false;
 }
 
 QPixmap *KVImageHolder::imagePix()
 {
+  if( m_empty )
+    return 0;
+
   if ( movie() )
     return &((QPixmap&)(movie()->framePixmap()));
     
   return pixmap();
 }
 
+void KVImageHolder::clear()
+{
+  setBackgroundColor( QColor( 0, 0, 0 ) );
+  resize( 0, 0 );
+  m_empty = true;
+}
+
 #include "canvas.moc"
 
