@@ -8,7 +8,7 @@
 *****************************************************************************/
 
 #include <qtabdialog.h>
-#include <qmultilinedit.h>
+#include <qmultilineedit.h>
 #include <qbuttongroup.h>
 #include <qradiobutton.h>
 #include <qlabel.h>
